<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model;

use Exception;
use FiloBlu\MappConnector\Adapter\RestAdapter;
use FiloBlu\MappConnector\Adapter\SftpAdapter;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Serialize\SerializerInterface;

/**
 * Class Config
 * @package FiloBlu\MappConnector\Model
 */
class Config
{

    /**
     * XML PATHS
     */
    const MAPPCONNECTOR_API_BASE_URL = 'mappconnector/connection/api_base_url';
    const MAPPCONNECTOR_API_SYSTEM_NAME = 'mappconnector/connection/api_system_name';
    const MAPPCONNECTOR_API_USERNAME = 'mappconnector/connection/api_username';
    const MAPPCONNECTOR_API_PASSWORD = 'mappconnector/connection/api_password';

    const MAPPCONNECTOR_FTP_URL = 'mappconnector/export/ftp_url';
    const MAPPCONNECTOR_FTP_LOGIN = 'mappconnector/export/ftp_login';
    const MAPPCONNECTOR_FTP_PASSWORD = 'mappconnector/export/ftp_password';
    const MAPPCONNECTOR_LOCAL_ROOT_DIR = 'mappconnector/export/local_root_dir';
    const MAPPCONNECTOR_REMOTE_ROOT_DIR = 'mappconnector/export/remote_root_dir';

    const MAPPCONNECTOR_GENDER_MAPPING = 'mappconnector/gender_mapping/gender';

    const MAPPCONNECTOR_NULL_PLACEHOLDER = '##NULL##';

    /**
     * @var ScopeConfigInterface
     */
    protected $_config;

    /**
     * @var
     */
    protected $_entity;

    /**
     * Config constructor.
     * @param ScopeConfigInterface $config
     */
    public function __construct(ScopeConfigInterface $config)
    {
        $this->_config = $config;
    }

    /**
     * @return mixed
     */
    public function getApiBaseUrl()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_BASE_URL);
    }

    /**
     * @return string
     */
    public function getApiPrefix()
    {
        return '/' . $this->_config->getValue(self::MAPPCONNECTOR_API_SYSTEM_NAME) . '/api/rest/v6';
    }

    /**
     * @return mixed
     */
    public function getApiUsername()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_USERNAME);
    }

    /**
     * @return mixed
     */
    public function getApiPassword()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_PASSWORD);
    }

    /**
     * @return mixed
     */
    public function getFTPUrl()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_URL);
    }

    /**
     * @return string
     */
    public function getFTPLogin()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_LOGIN);
    }

    /**
     * @return mixed
     */
    public function getFTPPassword()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_PASSWORD);
    }

    /**
     * @return mixed
     */
    public function getLocalRootDir()
    {
        $path = str_replace('\\', '/', $this->_config->getValue(self::MAPPCONNECTOR_LOCAL_ROOT_DIR));
        return str_replace('/', DIRECTORY_SEPARATOR, trim($path, '/'));
    }

    /**
     * @return mixed
     */
    public function getRemoteRootDir()
    {
        $path = $this->_config->getValue(self::MAPPCONNECTOR_REMOTE_ROOT_DIR);

        if ($path === null) {
            return '';
        }

        return trim(trim($path, '/'));
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getDomainsToExclude()
    {
        return array_map('trim', explode(',', $this->getConfigValue('mappconnector/general/exclude_mail')));
    }

    /**
     * @param $config_path
     * @param bool $unserialize
     * @return mixed
     * @throws Exception
     */
    public function getConfigValue($config_path, $unserialize = false)
    {
        return (!$unserialize) ? $this->_config->getValue($config_path) : $this->unserializeConfigData($this->_config->getValue($config_path));
    }

    /**
     * Fix unserialize from different version of magento
     *
     * @param $data
     * @return mixed
     * @throws Exception
     */
    public function unserializeConfigData($data)
    {
        $serialized = $this->checkIfIsSerialized($data);

        if (class_exists(SerializerInterface::class)) {
            $objectManager = ObjectManager::getInstance();
            $serializer = $objectManager->create(SerializerInterface::class);
            if ($serialized) {
                return $serializer->unserialize($data);
            } else {
                return $serializer->jsonDecode($data);
            }
        }
        if ($serialized) {
            return unserialize($data);
        } else {
            return json_decode($data);
        }
    }

    /**
     * Check if is serialized
     *
     * @param $data
     * @return bool
     * @throws Exception
     */
    public function checkIfIsSerialized($data)
    {
        //Is Json
        json_decode($data);

        if (json_last_error() === JSON_ERROR_NONE) {
            return false;
        }

        //Is Serialized
        $res = @unserialize($data);
        if ($res !== false) {
            return true;
        }

        //Unknow format
        throw new Exception('Unable to unserialize data from config. Unknown format');
    }

    /**
     * @param $entity
     */
    public function setEntity($entity)
    {
        $this->_entity = $entity;
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getNormalizedConfigObject()
    {
        $config = [];
        $c = $this->getConfigValue($this->getEntityXmlPath(), true);
        foreach ($c as $v) {
            $v = (array)$v;
            $config[$v[$this->_entity]] = $v["{$this->_entity}_mapp_field"];
        }
        return $config;
    }

    /**
     * @return string
     */
    public function getEntityXmlPath()
    {
        $en = explode('_', $this->_entity);
        return "mappconnector/{$en[0]}_mapping/{$this->_entity}";
    }

    /**
     * @param $gender
     * @return mixed
     * @throws Exception
     */
    public function getNormalizedGender($gender)
    {
        $genderMapping = $this->getNormalizedConfigObjectByPath(self::MAPPCONNECTOR_GENDER_MAPPING, 'gender');

        if (count($genderMapping) > 0) {
            //get placeholder value for empty or null values
            if (array_key_exists(self::MAPPCONNECTOR_NULL_PLACEHOLDER, $genderMapping) && ($gender === null || $gender === '')) {
                $gender = $genderMapping[self::MAPPCONNECTOR_NULL_PLACEHOLDER];
            } //return mapped array
            else {
                $gender = (isset($genderMapping[$gender]) ? $genderMapping[$gender] : $gender);
            }
        }

        return $gender;
    }

    /**
     * @param $path
     * @param $entity
     * @return array
     * @throws Exception
     */
    public function getNormalizedConfigObjectByPath($path, $entity)
    {
        $config = [];
        $c = $this->getConfigValue($path, true);
        foreach ($c as $v) {
            $v = (array)$v;
            $config[$v[$entity]] = $v["{$entity}_mapp_field"];
        }
        return $config;
    }

    /**
     * @param $map
     * @param $array
     * @return array
     */
    public function getMappedArray($map, $array)
    {
        $result = [];
        foreach ($array as $k => $v) {
            if (in_array($k, array_keys($map))) {
                if (is_array($v)) {
                    $v = implode(',', $v);
                }
                $result[$map[$k]] = $v;
            }
        }
        return $result;
    }

    /**
     * @param $adapter
     * @return mixed
     */
    public function getAdapter($adapter)
    {
        $adapterObject = [
            'rest' => RestAdapter::class,
            'sftp' => SftpAdapter::class
        ];
        return $adapterObject[$adapter];
    }
}
